<?php
/**
 * Dropfiles
 *
 * We developed this code with our hearts and passion.
 * We hope you found it useful, easy to understand and to customize.
 * Otherwise, please feel free to contact us at contact@joomunited.com *
 * @package Dropfiles
 * @copyright Copyright (C) 2013 JoomUnited (http://www.joomunited.com). All rights reserved.
 * @copyright Copyright (C) 2013 Damien Barrère (http://www.crac-design.com). All rights reserved.
 * @license GNU General Public License version 2 or later; http://www.gnu.org/licenses/gpl-2.0.html
 *
 */

// no direct access
defined('_JEXEC') or die;
$path_admin_categories = JPATH_ROOT . DIRECTORY_SEPARATOR . 'administrator' . DIRECTORY_SEPARATOR . 'components';
$path_admin_categories .= DIRECTORY_SEPARATOR . 'com_categories' . DIRECTORY_SEPARATOR . 'controllers';
$path_admin_categories .= DIRECTORY_SEPARATOR . 'categories.php';
require_once($path_admin_categories);

jimport('joomla.filesystem.folder');

/**
 * Class DropfilesControllerGoogledrive
 */
class DropfilesControllerGoogledrive extends CategoriesControllerCategories
{
    /**
     * Proxy for getModel
     *
     * @param string $name
     * @param string $prefix
     * @param array $config
     * @return JModelLegacy
     *
     */
    public function getModel($name = 'Category', $prefix = 'DropfilesModel', $config = array('ignore_request' => true))
    {
        $model = parent::getModel($name, $prefix, $config);
        return $model;
    }


    /**
     * Get authorize Url
     */
    public function getAuthorizeUrl()
    {
        $google = new DropfilesGoogle();
        $url = $google->getAuthorisationUrl();
        $this->setRedirect($url);
        $this->redirect();
    }

    /**
     * Authenticate GGD
     */
    public function authenticate()
    {

        $canDo = DropfilesHelper::getActions();

        if ($canDo->get('core.admin')) {
            $google = new DropfilesGoogle();
            $credentials = $google->authenticate();
            $google->storeCredentials($credentials);

            //Check if dropfiles folder exists and create if not
            $params = JComponentHelper::getParams('com_dropfiles');
            if (!$google->folderExists($params->get('google_base_folder', null))) {
                $folder = $google->createFolder('Dropfiles - ' . JFactory::getApplication()->getCfg('sitename'));
                DropfilesComponentHelper::setParams(array('google_base_folder' => $folder->id));
            }

            $this->setRedirect('index.php?option=com_dropfiles&view=googledrive&layout=redirect');
            $this->redirect();
        } else {
            $redirectDropfiles = 'index.php?option=com_dropfiles&view=dropfiles';
            $this->setRedirect($redirectDropfiles, JText::_('JLIB_APPLICATION_ERROR_SAVE_NOT_PERMITTED'));
            $this->redirect();
        }
    }

    /**
     * Check auth
     * @return bool
     *
     */
    public function checkauth()
    {
        $canDo = DropfilesHelper::getActions();
        if ($canDo->get('core.admin')) {
            $google = new DropfilesGoogle();
            return $google->checkAuth();
        } else {
            $redirectDropfiles = 'index.php?option=com_dropfiles&view=dropfiles';
            $this->setRedirect($redirectDropfiles, JText::_('JLIB_APPLICATION_ERROR_SAVE_NOT_PERMITTED'));
            $this->redirect();
        }
    }

    /**
     * Logout google
     */
    public function logout()
    {
        $canDo = DropfilesHelper::getActions();

        if ($canDo->get('core.admin')) {
            $google = new DropfilesGoogle();
            $google->logout();

            DropfilesComponentHelper::setParams(array('google_base_folder' => ''));
            DropfilesComponentHelper::setParams(array('google_credentials' => ''));

            $this->setRedirect($_SERVER['HTTP_REFERER']);
            $this->redirect();
        } else {
            $redirectDropfiles = 'index.php?option=com_dropfiles&view=dropfiles';
            $this->setRedirect($redirectDropfiles, JText::_('JLIB_APPLICATION_ERROR_SAVE_NOT_PERMITTED'));
            $this->redirect();
        }
    }


    /**
     * List folder cloud in Dropfiles
     *
     * @param $categories
     * @return array
     *
     */
    public function foldersCloudInDropfiles($categories)
    {
        $catCloud = array();
        if (isset($categories)) {
            foreach ($categories as $category) {
                $thisModel = $this->getModel('categories');
                $oneCat = $thisModel->getOneCatByLocalId($category->parent_id);
                $parent_cloud_id = 1;
                if (isset($oneCat->cloud_id)) {
                    $parent_cloud_id = $oneCat->cloud_id;
                }
                $catCloud[$category->cloud_id] = array('id' => $category->id, 'title' => $category->title,
                    'parent_id' => $category->parent_id, 'parent_cloud_id' => $parent_cloud_id);
            }
        }
        return $catCloud;
    }

    /**
     * List folder cloud in Google drive
     *
     * @return array|bool
     *
     */
    public function foldersCloudInGoogleDrive()
    {
        $params = JComponentHelper::getParams('com_dropfiles');
        $dropfilesGoogle = new DropfilesGoogle();
        try {
            $lstFolder = $dropfilesGoogle->getListFolder($params->get('google_base_folder'));
        } catch (Exception $e) {
            if ($params->get('sync_log_option') == 1) {
                $erros = $e->getMessage() . $e->getTraceAsString() . PHP_EOL;
                JLog::add($erros, JLog::ERROR, 'com_dropfiles');
            }
            $lstFolder = false;
        }
        return $lstFolder;
    }

    /**
     * Google sync
     */
    public function googlesync($cron = false)
    {
        $params = JComponentHelper::getParams('com_dropfiles');
        if ($params->get('sync_log_option') != null && $params->get('sync_log_option') == 1) {
            JLog::addLogger(
                array(
                    // Sets file name
                    'text_file' => 'com_dropfiles.synclog.php',
                    // Sets the format of each line
                    'text_entry_format' => '{DATETIME} {PRIORITY} {CLIENTIP} {MESSAGE}'
                ),
                // Sets all but DEBUG log level messages to be sent to the file
                JLog::ALL & ~JLog::DEBUG,
                // The log category which should be recorded in this file
                array('com_dropfiles')
            );
        }
        if ($params->get('google_credentials') != null) {
            $thisModel = $this->getModel('categories');
            //folders in Dropfiles
            $folderCloudInDropfiles = $this->foldersCloudInDropfiles($thisModel->getAllCat());
            //folders in Google Drive
            $folderCloudInGoogleDrive = $this->foldersCloudInGoogleDrive();
            if ($params->get('sync_log_option') == 1) {
                $erros = "folderCloudInGoogleDrive - ";
                $erros .= json_encode($folderCloudInGoogleDrive) . PHP_EOL;
                $erros .= "folderCloudInDropfiles - ";
                $erros .= json_encode($folderCloudInDropfiles) . PHP_EOL;
                JLog::add($erros, JLog::INFO, 'com_dropfiles');
            }

            //folders created in Google Drive without have in Dropfiles
            $folders_diff = array();
            $folders_diff_del = array();
            if ($folderCloudInGoogleDrive !== false) {  //to ensure there isn't error when connect with GD
                if (count($folderCloudInDropfiles) > 0) {
                    $folders_diff = array_diff_key($folderCloudInGoogleDrive, $folderCloudInDropfiles);
                    $folders_diff_del = array_diff_key($folderCloudInDropfiles, $folderCloudInGoogleDrive);
                    if ($params->get('sync_log_option') == 1) {
                        $erros = "folders_diff - ";
                        $erros .= json_encode($folders_diff) . PHP_EOL;
                        $erros .= "folders_diff_del - ";
                        $erros .= json_encode($folders_diff_del) . PHP_EOL;
                        JLog::add($erros, JLog::INFO, 'com_dropfiles');
                    }
                    foreach ($folderCloudInDropfiles as $k => $v) {
                        $objectCurrent = $thisModel->getOneCatByCloudId($k);
                        if ((!empty($folderCloudInGoogleDrive) && isset($folderCloudInGoogleDrive[$k])) &&
                            $folderCloudInDropfiles[$k]['title'] != $folderCloudInGoogleDrive[$k]['title']) {
                            try {
                                $thisModel->updateTitleById($objectCurrent->id, $folderCloudInGoogleDrive[$k]['title']);
                            } catch (Exception $e) {
                                $erros = "updateTitleById-Exception: " . $e->getMessage();
                                JLog::add($erros, JLog::ERROR, 'com_dropfiles');
                            }
                        }
                        if (!empty($folderCloudInGoogleDrive) && isset($folderCloudInGoogleDrive[$k])) {
                            // Fix Sync Children from google drive to dropfiles
                            $localParentId = $folderCloudInDropfiles[$k]['parent_id'];
                            $googleParentId = $folderCloudInGoogleDrive[$k]['parent_id'];

                            if (is_string($googleParentId)) {
                                // Get Local Id from google CloudID
                                $googleLocalParams = $thisModel->getOneCatByCloudId($googleParentId);
                                $googleLocalParentId = $googleLocalParams->id;

                                if ($localParentId != $googleLocalParentId) {
                                    try {
                                        $this->order('first-child', $objectCurrent->id, $googleLocalParentId);
                                    } catch (Exception $e) {
                                        $erros = "ChangeLocalParentFromCloud-Exception: " . $e->getMessage();
                                        JLog::add($erros, JLog::ERROR, 'com_dropfiles');
                                    }
                                }
                            }
                        }
                    }
                } else {
                    $folders_diff = $folderCloudInGoogleDrive;
                }
            }

            if (count($folders_diff_del) > 0) {
                foreach ($folders_diff_del as $CloudIdDel => $folderDataDel) {
                    $catInfoLocal = $thisModel->getOneCatByCloudId($CloudIdDel);
                    $thisModel->deleteOnDropfiles($CloudIdDel);
                    $thisModel->deleteOnCategories($catInfoLocal->id);
                }
            }
            //if exists diff key array
            if (count($folders_diff) > 0) {
                $lstCloudIdOnDropfiles = $thisModel->arrayCloudIdDropfiles();
                foreach ($folders_diff as $CloudId => $folderData) {
                    try {
                        //if has parent_id
                        if (is_string($folderData['parent_id'])) {
                            $check = in_array($folderData['parent_id'], $lstCloudIdOnDropfiles);
                            if (!$check) {
                                //create Parent New
                                //$ParentCloudInfo = $dropfilesGoogle->getFileInfos($folderData['parent_id']);
                                $ParentCloudInfo = $folderCloudInGoogleDrive[$folderData['parent_id']];
                                $newCatId = $thisModel->createOnCategories($ParentCloudInfo['title'], 1, 1);
                                if ($newCatId) {
                                    $thisModel->createOnDropfiles($newCatId, 'googledrive', $folderData['parent_id']);
                                    $lstCloudIdOnDropfiles[] = $folderData['parent_id'];
                                }
                                //create Children New with parent_id in dropfiles
                                if ($newCatId) {
                                    $catRecentCreate = $thisModel->getOneCatByLocalId($newCatId);
                                    $newChildId = $thisModel->createOnCategories(
                                        $folderData['title'],
                                        $catRecentCreate->id,
                                        (int)$catRecentCreate->level + 1
                                    );
                                    if ($newChildId) {
                                        $thisModel->createOnDropfiles(
                                            $newChildId,
                                            'googledrive',
                                            $CloudId
                                        );
                                        $lstCloudIdOnDropfiles[] = $CloudId;
                                    }
                                }
                            } else {
                                //create Children New with parent_id in dropfiles
                                $catOldInfo = $thisModel->getOneCatByCloudId($folderData['parent_id']);
                                $newCatId = $thisModel->createOnCategories(
                                    $folderData['title'],
                                    $catOldInfo->id,
                                    (int)$catOldInfo->level + 1
                                );
                                if ($newCatId) {
                                    $thisModel->createOnDropfiles(
                                        $newCatId,
                                        'googledrive',
                                        $CloudId
                                    );
                                    $lstCloudIdOnDropfiles[] = $CloudId;
                                }
                            }
                        } else {
                            //create Folder New
                            $newCatId = $thisModel->createOnCategories($folderData['title'], 1, 1);
                            if ($newCatId) {
                                $thisModel->createOnDropfiles($newCatId, 'googledrive', $CloudId);
                                $lstCloudIdOnDropfiles[] = $CloudId;
                            }
                        }
                    } catch (Exception $e) {
                        $erros = $e->getMessage();
                        JLog::add($erros, JLog::ERROR, 'com_dropfiles');
                        break;
                    }
                }
            }
            JLoader::register(
                'DropfilesComponentHelper',
                JPATH_ADMINISTRATOR . '/components/com_dropfiles/helpers/component.php'
            );
            $clsDropfilesHelper = new DropfilesComponentHelper();
            $clsDropfilesHelper->setParams(array('last_log' => date('Y-m-d H:i:s')));
            if (!$cron) {
                echo json_encode(array('status' => true));
                JFactory::getApplication()->close();
            }
        }
    }

    /**
     * Sync Google Drive on crontask
     *
     * @return void
     */
    public function googlesyncviacron()
    {
        $this->googlesync(true);
        $step = 0;
        $path_drf_google = JPATH_BASE . '/components/com_dropfiles/controllers/frontgoogle.php';
        require_once $path_drf_google;
        $frontgg = new DropfilesControllerFrontgoogle();
        while ($frontgg->syncFiles(true, $step)) {
            $step++;
        }


        echo json_encode(array('success' => true));
        JFactory::getApplication()->close();
    }

    /**
     * Order item
     *
     * @param $position
     * @param $pk
     * @param $ref
     *
     */
    public function order($position, $pk, $ref)
    {
        $model = $this->getModel();
        $canDo = DropfilesHelper::getActions();
        if (!$canDo->get('core.edit')) {
            if ($canDo->get('core.edit.own')) {
                $category = $model->getItem($pk);
                if ($category->created_user_id != JFactory::getUser()->id) {
                    $this->exitStatus('not permitted');
                }
            } else {
                $this->exitStatus('not permitted');
            }
        }

        if ($ref == 0) {
            $ref = 1;
        }
        if ($position !== 'after') {
            $position = 'first-child';
        }

        $table = $model->getTable();
        if ($table->moveByReference($ref, $position, $pk)) {
            $this->exitStatus(true, $pk . ' ' . $position . ' ' . $ref);
        }
        $this->exitStatus(JText::_('COM_DROPFILES_CTRL_MESSAGE_ERROR'));
    }

    /**
     * Return a json response
     * @param $status
     * @param array $datas array of datas to return with the json string
     *
     */
    private function exitStatus($status, $datas = array())
    {
        $response = array('response' => $status, 'datas' => $datas);
        echo json_encode($response);
        JFactory::getApplication()->close();
    }
}
