<?php
/**
 * Dropfiles
 *
 * We developed this code with our hearts and passion.
 * We hope you found it useful, easy to understand and to customize.
 * Otherwise, please feel free to contact us at contact@joomunited.com *
 * @package Dropfiles
 * @copyright Copyright (C) 2013 JoomUnited (http://www.joomunited.com). All rights reserved.
 * @copyright Copyright (C) 2013 Damien Barrère (http://www.crac-design.com). All rights reserved.
 * @license GNU General Public License version 2 or later; http://www.gnu.org/licenses/gpl-2.0.html
 * @since        1.6
 */


defined('_JEXEC') or die;

/**
 * @package        Joomla.Administrator
 * @subpackage    com_messages
 * @since        1.6
 */
class DropfilesCloudHelper
{
    /*----------- OneDrive -----------------*/

    /**
     * All config onedrive
     * @return array
     *
     */
    private static function oneDriveConfigs()
    {
        $default = array(
            'onedriveKey' => '',
            'onedriveSecret' => '',
            'onedriveCredentials' => '',
            'onedriveSyncTime' => '5',
            'onedriveSyncMethod' => 'sync_page_curl',
            'onedrive_last_log' => '',
            'onedriveBaseFolderId' => 0,
            'onedriveBaseFolderName' => '');
        return $default;
    }

    /**
     * All config old
     * @return array
     *
     */
    private static function oneDriveConfigsOld()
    {
        $default = array(
            'onedriveKeyOld' => '',
            'onedriveSecretOld' => '',
            'onedriveCredentialsOld' => '',
            'onedriveBaseFolderIdOld' => '',
            'onedriveBaseFolderNameOld' => ''
        );
        return $default;
    }

    /**
     * Get all config params of onedrive
     *
     * @return array
     *
     */
    public static function getAllOneDriveConfigs()
    {
        $default = self::getParamsOneDrive(self::oneDriveConfigs());
        return $default;
    }

    /**
     * Save config onedriver
     *
     * @param $data
     * @return bool
     *
     */
    public static function setParamsConfigs($data)
    {
        return DropfilesComponentHelper::setParams($data);
    }

    /**
     *  Get all old config params of onedrive
     *
     * @return array
     *
     */
    public static function getAllOneDriveConfigsOld()
    {
        $default = self::getParamsOneDrive(self::oneDriveConfigsOld());
        return $default;
    }

    /**
     * Save old config onedrive
     * @param $data
     * @return bool
     *
     */
    public static function setParamsConfigsOld($data)
    {
        $paramOld = self::oneDriveConfigsOld();
        foreach ($data as $key => $val) {
            if (array_key_exists($key . 'Old', $paramOld)) {
                $paramOld[$key . 'Old'] = $val;
            }
        }
        return DropfilesComponentHelper::setParams($paramOld);
    }

    /**
     * Get params OneDrive
     *
     * @param $array
     * @return array
     *
     */
    public static function getParamsOneDrive($array)
    {
        $lis_params = array();

        $db = JFactory::getDbo();
        $query = $db->getQuery(true);
        $query->select('params')->from('#__extensions')->where('element = "com_dropfiles"');
        $db->setQuery((string) $query);
        $params = new JRegistry;
        if ($results = $db->loadObject()) {
            $params->loadString($results->params, 'JSON');
        }

        foreach ($array as $key => $val) {
            if (!is_numeric($key)) {
                $lis_params[$key] = $params->get($key, $val);
            } else {
                $lis_params[$val] = $params->get($val);
            }
        }
        return $lis_params;
    }

    /**
     * Get data config by OneDrive
     *
     * @param $name
     * @return array|null
     *
     */
    public static function getDataConfigByOneDrive($name)
    {
        $OneDriveParams = array();
        if (self::getAllOneDriveConfigs()) {
            foreach (self::getAllOneDriveConfigs() as $key => $val) {
                if (strpos($key, 'onedrive') !== false) {
                    $OneDriveParams[$key] = $val;
                }
            }
            $result = null;
            switch ($name) {
                case 'onedrive':
                    $result = $OneDriveParams;
                    break;
            }
            return $result;
        }
    }

    /**
     * Replace special characters Id OneDrive
     * @param $id
     * @param bool $rplSpecial
     * @return mixed
     *
     */
    public static function replaceIdOneDrive($id, $rplSpecial = true)
    {
        if ($rplSpecial) {
            return str_replace("!", "-", $id);
        } else {
            return str_replace("-", "!", $id);
        }
    }

    /**
     * Strips the last extension off of a file name
     *
     * @param   string $file The file name
     *
     * @return  string  The file name without the extension
     *
     * @since   11.1
     */
    public static function stripExt($file)
    {
        return preg_replace('#\.[^.]*$#', '', $file);
    }

    /**
     * Get ext of file
     *
     * @param $file
     * @return bool|string
     *
     */
    public static function getExt($file)
    {
        $dot = strrpos($file, '.') + 1;

        return substr($file, $dot);
    }
}
